from .__init__ import ctypes, typing, abc, SDL_POINTER, \
    SDL_FUNC_TYPE, SDL_FUNC, SDL_TYPE, SDL_BINARY, SDL_ENUM

from .SDL_video import SDL_Window, SDL_WindowID, SDL_DisplayID
from .SDL_sensor import SDL_SensorID
from .SDL_joystick import SDL_JoystickID
from .SDL_keyboard import SDL_KeyboardID
from .SDL_keycode import SDL_Keycode, SDL_Keymod
from .SDL_scancode import SDL_Scancode
from .SDL_mouse import SDL_MouseID, SDL_MouseWheelDirection, SDL_MouseButtonFlags
from .SDL_power import SDL_PowerState
from .SDL_audio import SDL_AudioDeviceID
from .SDL_camera import SDL_CameraID
from .SDL_pen import SDL_PenID, SDL_PenInputFlags, SDL_PenAxis
from .SDL_touch import SDL_TouchID, SDL_FingerID

SDL_EventType: typing.TypeAlias = SDL_TYPE["SDL_EventType", SDL_ENUM]

SDL_EVENT_FIRST: int = 0x0

SDL_EVENT_QUIT, SDL_EVENT_TERMINATING, SDL_EVENT_LOW_MEMORY, SDL_EVENT_WILL_ENTER_BACKGROUND, \
    SDL_EVENT_DID_ENTER_BACKGROUND, SDL_EVENT_WILL_ENTER_FOREGROUND, SDL_EVENT_DID_ENTER_FOREGROUND, \
        SDL_EVENT_LOCALE_CHANGED, SDL_EVENT_SYSTEM_THEME_CHANGED = range(0x100, 0x109)

SDL_EVENT_DISPLAY_ORIENTATION, SDL_EVENT_DISPLAY_ADDED, SDL_EVENT_DISPLAY_REMOVED, SDL_EVENT_DISPLAY_MOVED, \
    SDL_EVENT_DISPLAY_DESKTOP_MODE_CHANGED, SDL_EVENT_DISPLAY_CURRENT_MODE_CHANGED, SDL_EVENT_DISPLAY_CONTENT_SCALE_CHANGED = range(0x151, 0x158)

SDL_EVENT_DISPLAY_FIRST, SDL_EVENT_DISPLAY_LAST = \
    SDL_EVENT_DISPLAY_ORIENTATION, SDL_EVENT_DISPLAY_CONTENT_SCALE_CHANGED

SDL_EVENT_WINDOW_SHOWN, SDL_EVENT_WINDOW_HIDDEN, SDL_EVENT_WINDOW_EXPOSED, SDL_EVENT_WINDOW_MOVED, SDL_EVENT_WINDOW_RESIZED, \
    SDL_EVENT_WINDOW_PIXEL_SIZE_CHANGED, SDL_EVENT_WINDOW_METAL_VIEW_RESIZED, SDL_EVENT_WINDOW_MINIMIZED, SDL_EVENT_WINDOW_MAXIMIZED, \
        SDL_EVENT_WINDOW_RESTORED, SDL_EVENT_WINDOW_MOUSE_ENTER, SDL_EVENT_WINDOW_MOUSE_LEAVE, SDL_EVENT_WINDOW_FOCUS_GAINED, SDL_EVENT_WINDOW_FOCUS_LOST, \
            SDL_EVENT_WINDOW_CLOSE_REQUESTED, SDL_EVENT_WINDOW_HIT_TEST, SDL_EVENT_WINDOW_ICCPROF_CHANGED, SDL_EVENT_WINDOW_DISPLAY_CHANGED, SDL_EVENT_WINDOW_DISPLAY_SCALE_CHANGED, \
                SDL_EVENT_WINDOW_SAFE_AREA_CHANGED, SDL_EVENT_WINDOW_OCCLUDED, SDL_EVENT_WINDOW_ENTER_FULLSCREEN, SDL_EVENT_WINDOW_LEAVE_FULLSCREEN, \
                    SDL_EVENT_WINDOW_DESTROYED, SDL_EVENT_WINDOW_HDR_STATE_CHANGED = range(0x202, 0x21B)

SDL_EVENT_WINDOW_FIRST, SDL_EVENT_WINDOW_LAST = \
    SDL_EVENT_WINDOW_SHOWN, SDL_EVENT_WINDOW_HDR_STATE_CHANGED

SDL_EVENT_KEY_DOWN, SDL_EVENT_KEY_UP, SDL_EVENT_TEXT_EDITING, SDL_EVENT_TEXT_INPUT, SDL_EVENT_KEYMAP_CHANGED, \
    SDL_EVENT_KEYBOARD_ADDED, SDL_EVENT_KEYBOARD_REMOVED, SDL_EVENT_TEXT_EDITING_CANDIDATES = range(0x300, 0x308)

SDL_EVENT_MOUSE_MOTION, SDL_EVENT_MOUSE_BUTTON_DOWN, SDL_EVENT_MOUSE_BUTTON_UP, \
    SDL_EVENT_MOUSE_WHEEL, SDL_EVENT_MOUSE_ADDED, SDL_EVENT_MOUSE_REMOVED = range(0x400, 0x406)

SDL_EVENT_JOYSTICK_AXIS_MOTION, SDL_EVENT_JOYSTICK_BALL_MOTION, SDL_EVENT_JOYSTICK_HAT_MOTION, SDL_EVENT_JOYSTICK_BUTTON_DOWN, SDL_EVENT_JOYSTICK_BUTTON_UP, \
    SDL_EVENT_JOYSTICK_ADDED, SDL_EVENT_JOYSTICK_REMOVED, SDL_EVENT_JOYSTICK_BATTERY_UPDATED, SDL_EVENT_JOYSTICK_UPDATE_COMPLETE = range(0x600, 0x609)

SDL_EVENT_GAMEPAD_AXIS_MOTION, SDL_EVENT_GAMEPAD_BUTTON_DOWN, SDL_EVENT_GAMEPAD_BUTTON_UP, SDL_EVENT_GAMEPAD_ADDED, SDL_EVENT_GAMEPAD_REMOVED, \
    SDL_EVENT_GAMEPAD_REMAPPED, SDL_EVENT_GAMEPAD_TOUCHPAD_DOWN, SDL_EVENT_GAMEPAD_TOUCHPAD_MOTION, SDL_EVENT_GAMEPAD_TOUCHPAD_UP, SDL_EVENT_GAMEPAD_SENSOR_UPDATE, \
        SDL_EVENT_GAMEPAD_UPDATE_COMPLETE, SDL_EVENT_GAMEPAD_STEAM_HANDLE_UPDATED = range(0x650, 0x65C)

SDL_EVENT_FINGER_DOWN, SDL_EVENT_FINGER_UP, SDL_EVENT_FINGER_MOTION, \
    SDL_EVENT_FINGER_CANCELED = range(0x700, 0x704)

SDL_EVENT_CLIPBOARD_UPDATE: int = 0x900

SDL_EVENT_DROP_FILE, SDL_EVENT_DROP_TEXT, SDL_EVENT_DROP_BEGIN, \
    SDL_EVENT_DROP_COMPLETE, SDL_EVENT_DROP_POSITION = range(0x1000, 0x1005)

SDL_EVENT_AUDIO_DEVICE_ADDED, SDL_EVENT_AUDIO_DEVICE_REMOVED, \
    SDL_EVENT_AUDIO_DEVICE_FORMAT_CHANGED = range(0x1100, 0x1103)

SDL_EVENT_SENSOR_UPDATE: int = 0x1200

SDL_EVENT_PEN_PROXIMITY_IN, SDL_EVENT_PEN_PROXIMITY_OUT, SDL_EVENT_PEN_DOWN, SDL_EVENT_PEN_UP, SDL_EVENT_PEN_BUTTON_DOWN, \
    SDL_EVENT_PEN_BUTTON_UP, SDL_EVENT_PEN_MOTION, SDL_EVENT_PEN_AXIS = range(0x1300, 0x1308)

SDL_EVENT_CAMERA_DEVICE_ADDED, SDL_EVENT_CAMERA_DEVICE_REMOVED, \
    SDL_EVENT_CAMERA_DEVICE_APPROVED, SDL_EVENT_CAMERA_DEVICE_DENIED = range(0x1400, 0x1404)

SDL_EVENT_RENDER_TARGETS_RESET, SDL_EVENT_RENDER_DEVICE_RESET, \
    SDL_EVENT_RENDER_DEVICE_LOST = range(0x2000, 0x2003)

SDL_EVENT_PRIVATE0, SDL_EVENT_PRIVATE1, SDL_EVENT_PRIVATE2, \
    SDL_EVENT_PRIVATE3 = range(0x4000, 0x4004)

SDL_EVENT_POLL_SENTINEL: int = 0x7F00
SDL_EVENT_USER: int = 0x8000
SDL_EVENT_LAST: int = 0xFFFF
SDL_EVENT_ENUM_PADDING: int = 0x7FFFFFFF

class SDL_CommonEvent(ctypes.Structure):
    _fields_ = [
        ("type", ctypes.c_uint32),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64)
    ]

class SDL_DisplayEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("displayID", SDL_DisplayID),
        ("data1", ctypes.c_int32),
        ("data2", ctypes.c_int32)
    ]

class SDL_WindowEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("windowID", SDL_WindowID),
        ("data1", ctypes.c_int32),
        ("data2", ctypes.c_int32)
    ]

class SDL_KeyboardDeviceEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("which", SDL_KeyboardID)
    ]

class SDL_KeyboardEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("windowID", SDL_WindowID),
        ("which", SDL_KeyboardID),
        ("scancode", SDL_Scancode),
        ("key", SDL_Keycode),
        ("mod", SDL_Keymod),
        ("raw", ctypes.c_uint16),
        ("down", ctypes.c_bool),
        ("repeat", ctypes.c_bool)
    ]

class SDL_TextEditingEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("windowID", SDL_WindowID),
        ("text", ctypes.c_char_p),
        ("start", ctypes.c_int32),
        ("length", ctypes.c_int32)
    ]

class SDL_TextEditingCandidatesEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("windowID", SDL_WindowID),
        ("candidates", SDL_POINTER[ctypes.c_char_p]),
        ("num_candidates", ctypes.c_int32),
        ("selected_candidate", ctypes.c_int32),
        ("horizontal", ctypes.c_bool),
        ("padding1", ctypes.c_uint8),
        ("padding2", ctypes.c_uint8),
        ("padding3", ctypes.c_uint8)
    ]

class SDL_TextInputEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("windowID", SDL_WindowID),
        ("text", ctypes.c_char_p)
    ]

class SDL_MouseDeviceEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("which", SDL_MouseID)
    ]

class SDL_MouseMotionEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("windowID", SDL_WindowID),
        ("which", SDL_MouseID),
        ("state", SDL_MouseButtonFlags),
        ("x", ctypes.c_float),
        ("y", ctypes.c_float),
        ("xrel", ctypes.c_float),
        ("yrel", ctypes.c_float)
    ]

class SDL_MouseButtonEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("windowID", SDL_WindowID),
        ("which", SDL_MouseID),
        ("button", ctypes.c_uint8),
        ("down", ctypes.c_bool),
        ("clicks", ctypes.c_uint8),
        ("padding", ctypes.c_uint8),
        ("x", ctypes.c_float),
        ("y", ctypes.c_float)
    ]

class SDL_MouseWheelEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("windowID", SDL_WindowID),
        ("which", SDL_MouseID),
        ("x", ctypes.c_float),
        ("y", ctypes.c_float),
        ("direction", SDL_MouseWheelDirection),
        ("mouse_x", ctypes.c_float),
        ("mouse_y", ctypes.c_float),
        ("integer_x", ctypes.c_int32),
        ("integer_y", ctypes.c_int32)
    ]

class SDL_JoyAxisEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("which", SDL_JoystickID),
        ("axis", ctypes.c_uint8),
        ("padding1", ctypes.c_uint8),
        ("padding2", ctypes.c_uint8),
        ("padding3", ctypes.c_uint8),
        ("value", ctypes.c_int16),
        ("padding4", ctypes.c_uint16)
    ]

class SDL_JoyBallEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("which", SDL_JoystickID),
        ("ball", ctypes.c_uint8),
        ("padding1", ctypes.c_uint8),
        ("padding2", ctypes.c_uint8),
        ("padding3", ctypes.c_uint8),
        ("xrel", ctypes.c_int16),
        ("yrel", ctypes.c_int16)
    ]

class SDL_JoyHatEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("which", SDL_JoystickID),
        ("hat", ctypes.c_uint8),
        ("value", ctypes.c_uint8),
        ("padding1", ctypes.c_uint8),
        ("padding2", ctypes.c_uint8)
    ]

class SDL_JoyButtonEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("which", SDL_JoystickID),
        ("button", ctypes.c_uint8),
        ("down", ctypes.c_bool),
        ("padding1", ctypes.c_uint8),
        ("padding2", ctypes.c_uint8)
    ]

class SDL_JoyDeviceEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("which", SDL_JoystickID)
    ]

class SDL_JoyBatteryEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("which", SDL_JoystickID),
        ("state", SDL_PowerState),
        ("percent", ctypes.c_int)
    ]

class SDL_GamepadAxisEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("which", SDL_JoystickID),
        ("axis", ctypes.c_uint8),
        ("padding1", ctypes.c_uint8),
        ("padding2", ctypes.c_uint8),
        ("padding3", ctypes.c_uint8),
        ("value", ctypes.c_int16),
        ("padding4", ctypes.c_uint16)
    ]

class SDL_GamepadButtonEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("which", SDL_JoystickID),
        ("button", ctypes.c_uint8),
        ("down", ctypes.c_bool),
        ("padding1", ctypes.c_uint8),
        ("padding2", ctypes.c_uint8)
    ]

class SDL_GamepadDeviceEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("which", SDL_JoystickID)
    ]

class SDL_GamepadTouchpadEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("which", SDL_JoystickID),
        ("touchpad", ctypes.c_int32),
        ("finger", ctypes.c_int32),
        ("x", ctypes.c_float),
        ("y", ctypes.c_float),
        ("pressure", ctypes.c_float)
    ]

class SDL_GamepadSensorEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("which", SDL_JoystickID),
        ("sensor", ctypes.c_int32),
        ("data", ctypes.c_float * 3),
        ("sensor_timestamp", ctypes.c_uint64)
    ]

class SDL_AudioDeviceEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("which", SDL_AudioDeviceID),
        ("recording", ctypes.c_bool),
        ("padding1", ctypes.c_uint8),
        ("padding2", ctypes.c_uint8),
        ("padding3", ctypes.c_uint8)
    ]

class SDL_CameraDeviceEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("which", SDL_CameraID)
    ]

class SDL_RenderEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("windowID", SDL_WindowID)
    ]

class SDL_TouchFingerEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("touchID", SDL_TouchID),
        ("fingerID", SDL_FingerID),
        ("x", ctypes.c_float),
        ("y", ctypes.c_float),
        ("dx", ctypes.c_float),
        ("dy", ctypes.c_float),
        ("pressure", ctypes.c_float),
        ("windowID", SDL_WindowID)
    ]

class SDL_PenProximityEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("windowID", SDL_WindowID),
        ("which", SDL_PenID)
    ]

class SDL_PenMotionEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("windowID", SDL_WindowID),
        ("which", SDL_PenID),
        ("pen_state", SDL_PenInputFlags),
        ("x", ctypes.c_float),
        ("y", ctypes.c_float)
    ]

class SDL_PenTouchEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("windowID", SDL_WindowID),
        ("which", SDL_PenID),
        ("pen_state", SDL_PenInputFlags),
        ("x", ctypes.c_float),
        ("y", ctypes.c_float),
        ("eraser", ctypes.c_bool),
        ("down", ctypes.c_bool)
    ]

class SDL_PenButtonEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("windowID", SDL_WindowID),
        ("which", SDL_PenID),
        ("pen_state", SDL_PenInputFlags),
        ("x", ctypes.c_float),
        ("y", ctypes.c_float),
        ("button", ctypes.c_uint8),
        ("down", ctypes.c_bool)
    ]

class SDL_PenAxisEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("windowID", SDL_WindowID),
        ("which", SDL_PenID),
        ("pen_state", SDL_PenInputFlags),
        ("x", ctypes.c_float),
        ("y", ctypes.c_float),
        ("axis", SDL_PenAxis),
        ("value", ctypes.c_float)
    ]

class SDL_DropEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("windowID", SDL_WindowID),
        ("x", ctypes.c_float),
        ("y", ctypes.c_float),
        ("source", ctypes.c_char_p),
        ("data", ctypes.c_char_p)
    ]

class SDL_ClipboardEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("owner", ctypes.c_bool),
        ("num_mime_types", ctypes.c_int32),
        ("mime_types", SDL_POINTER[ctypes.c_char_p])
    ]

class SDL_SensorEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("which", SDL_SensorID),
        ("data", ctypes.c_float * 6),
        ("sensor_timestamp", ctypes.c_uint64)
    ]

class SDL_QuitEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64)
    ]

class SDL_UserEvent(ctypes.Structure):
    _fields_ = [
        ("type", ctypes.c_uint32),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("windowID", SDL_WindowID),
        ("code", ctypes.c_int32),
        ("data1", ctypes.c_void_p),
        ("data2", ctypes.c_void_p)
    ]

class SDL_Event(ctypes.Union):
    _fields_ = [
        ("type", ctypes.c_uint32),
        ("common", SDL_CommonEvent),
        ("display", SDL_DisplayEvent),
        ("window", SDL_WindowEvent),
        ("kdevice", SDL_KeyboardDeviceEvent),
        ("key", SDL_KeyboardEvent),
        ("edit", SDL_TextEditingEvent),
        ("edit_candidates", SDL_TextEditingCandidatesEvent),
        ("text", SDL_TextInputEvent),
        ("mdevice", SDL_MouseDeviceEvent),
        ("motion", SDL_MouseMotionEvent),
        ("button", SDL_MouseButtonEvent),
        ("wheel", SDL_MouseWheelEvent),
        ("jdevice", SDL_JoyDeviceEvent),
        ("jaxis", SDL_JoyAxisEvent),
        ("jball", SDL_JoyBallEvent),
        ("jhat", SDL_JoyHatEvent),
        ("jbutton", SDL_JoyButtonEvent),
        ("jbattery", SDL_JoyBatteryEvent),
        ("gdevice", SDL_GamepadDeviceEvent),
        ("gaxis", SDL_GamepadAxisEvent),
        ("gbutton", SDL_GamepadButtonEvent),
        ("gtouchpad", SDL_GamepadTouchpadEvent),
        ("gsensor", SDL_GamepadSensorEvent),
        ("adevice", SDL_AudioDeviceEvent),
        ("cdevice", SDL_CameraDeviceEvent),
        ("sensor", SDL_SensorEvent),
        ("quit", SDL_QuitEvent),
        ("user", SDL_UserEvent),
        ("tfinger", SDL_TouchFingerEvent),
        ("pproximity", SDL_PenProximityEvent),
        ("ptouch", SDL_PenTouchEvent),
        ("pmotion", SDL_PenMotionEvent),
        ("pbutton", SDL_PenButtonEvent),
        ("paxis", SDL_PenAxisEvent),
        ("render", SDL_RenderEvent),
        ("drop", SDL_DropEvent),
        ("clipboard", SDL_ClipboardEvent),
        ("padding", ctypes.c_uint8 * 128)
    ]

SDL_PumpEvents: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_PumpEvents", None, [], SDL_BINARY]

SDL_EventAction: typing.TypeAlias = SDL_TYPE["SDL_EventAction", SDL_ENUM]

SDL_ADDEVENT, SDL_PEEKEVENT, SDL_GETEVENT = range(3)

SDL_PeepEvents: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_PeepEvents", ctypes.c_int, [SDL_POINTER[SDL_Event], ctypes.c_int, SDL_EventAction, ctypes.c_uint32, ctypes.c_uint32], SDL_BINARY]
SDL_HasEvent: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_HasEvent", ctypes.c_bool, [ctypes.c_uint32], SDL_BINARY]
SDL_HasEvents: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_HasEvents", ctypes.c_bool, [ctypes.c_uint32, ctypes.c_uint32], SDL_BINARY]
SDL_FlushEvent: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_FlushEvent", None, [ctypes.c_uint32], SDL_BINARY]
SDL_FlushEvents: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_FlushEvents", None, [ctypes.c_uint32, ctypes.c_uint32], SDL_BINARY]
SDL_PollEvent: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_PollEvent", ctypes.c_bool, [SDL_POINTER[SDL_Event]], SDL_BINARY]
SDL_WaitEvent: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_WaitEvent", ctypes.c_bool, [SDL_POINTER[SDL_Event]], SDL_BINARY]
SDL_WaitEventTimeout: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_WaitEventTimeout", ctypes.c_bool, [SDL_POINTER[SDL_Event], ctypes.c_int32], SDL_BINARY]
SDL_PushEvent: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_PushEvent", ctypes.c_bool, [SDL_POINTER[SDL_Event]], SDL_BINARY]

SDL_EventFilter: typing.TypeAlias = SDL_FUNC_TYPE["SDL_EventFilter", ctypes.c_bool, [ctypes.c_void_p, SDL_POINTER[SDL_Event]]]

SDL_SetEventFilter: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetEventFilter", None, [SDL_EventFilter, ctypes.c_void_p], SDL_BINARY]
SDL_GetEventFilter: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetEventFilter", ctypes.c_bool, [SDL_POINTER[SDL_EventFilter], SDL_POINTER[ctypes.c_void_p]], SDL_BINARY]
SDL_AddEventWatch: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_AddEventWatch", ctypes.c_bool, [SDL_EventFilter, ctypes.c_void_p], SDL_BINARY]
SDL_RemoveEventWatch: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_RemoveEventWatch", None, [SDL_EventFilter, ctypes.c_void_p], SDL_BINARY]
SDL_FilterEvents: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_FilterEvents", None, [SDL_EventFilter, ctypes.c_void_p], SDL_BINARY]
SDL_SetEventEnabled: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetEventEnabled", None, [ctypes.c_uint32, ctypes.c_bool], SDL_BINARY]
SDL_EventEnabled: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_EventEnabled", ctypes.c_bool, [ctypes.c_uint32], SDL_BINARY]
SDL_RegisterEvents: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_RegisterEvents", ctypes.c_uint32, [ctypes.c_int], SDL_BINARY]
SDL_GetWindowFromEvent: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetWindowFromEvent", SDL_POINTER[SDL_Window], [SDL_POINTER[SDL_Event]], SDL_BINARY]


from .__init__ import sdl3, TEST_RegisterFunction

@TEST_RegisterFunction(["Darwin", "Windows", "Linux"])
def TEST_SDL_GetVersion() -> None:
    assert (_ := sdl3.SDL_GetVersion()), "Failed to get version."
    assert (major := sdl3.SDL_VERSIONNUM_MAJOR(_)) == sdl3.SDL_MAJOR_VERSION, f"Major version mismatch: {major}."
    assert (minor := sdl3.SDL_VERSIONNUM_MINOR(_)) == sdl3.SDL_MINOR_VERSION, f"Minor version mismatch: {minor}."
    assert (micro := sdl3.SDL_VERSIONNUM_MICRO(_)) == sdl3.SDL_MICRO_VERSION, f"Micro version mismatch: {micro}."
    assert sdl3.SDL_VERSION_ATLEAST(sdl3.SDL_MAJOR_VERSION, sdl3.SDL_MINOR_VERSION, sdl3.SDL_MICRO_VERSION), "Version is not at least the current version."
    assert _ == sdl3.SDL_VERSION, f"Version mismatch: {_}."

@TEST_RegisterFunction(["Darwin", "Windows", "Linux"])
def TEST_SDL_GetRevision() -> None:
    assert (_ := sdl3.SDL_GetRevision()), "Failed to get revision."
    assert _.decode(), "Revision is empty."


from .__init__ import ctypes, typing, abc, \
    SDL_ENUM, SDL_FUNC, SDL_TYPE, SDL_BINARY

SDL_BlendMode: typing.TypeAlias = SDL_TYPE["SDL_BlendMode", ctypes.c_uint32]

SDL_BLENDMODE_NONE: int = 0x00000000
SDL_BLENDMODE_BLEND: int = 0x00000001
SDL_BLENDMODE_BLEND_PREMULTIPLIED: int = 0x00000010
SDL_BLENDMODE_ADD: int = 0x00000002
SDL_BLENDMODE_ADD_PREMULTIPLIED: int = 0x00000020
SDL_BLENDMODE_MOD: int = 0x00000004
SDL_BLENDMODE_MUL: int = 0x00000008
SDL_BLENDMODE_INVALID: int = 0x7FFFFFFF

SDL_BlendOperation: typing.TypeAlias = SDL_TYPE["SDL_BlendOperation", SDL_ENUM]

SDL_BLENDOPERATION_ADD, SDL_BLENDOPERATION_SUBTRACT, SDL_BLENDOPERATION_REV_SUBTRACT, \
    SDL_BLENDOPERATION_MINIMUM, SDL_BLENDOPERATION_MAXIMUM = range(1, 6)

SDL_BlendFactor: typing.TypeAlias = SDL_TYPE["SDL_BlendFactor", SDL_ENUM]

SDL_BLENDFACTOR_ZERO, SDL_BLENDFACTOR_ONE, SDL_BLENDFACTOR_SRC_COLOR, SDL_BLENDFACTOR_ONE_MINUS_SRC_COLOR, \
    SDL_BLENDFACTOR_SRC_ALPHA, SDL_BLENDFACTOR_ONE_MINUS_SRC_ALPHA, SDL_BLENDFACTOR_DST_COLOR, SDL_BLENDFACTOR_ONE_MINUS_DST_COLOR, \
        SDL_BLENDFACTOR_DST_ALPHA, SDL_BLENDFACTOR_ONE_MINUS_DST_ALPHA = range(1, 11)

SDL_ComposeCustomBlendMode: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_ComposeCustomBlendMode", SDL_BlendMode, [SDL_BlendFactor, SDL_BlendFactor, SDL_BlendOperation, SDL_BlendFactor, SDL_BlendFactor, SDL_BlendOperation], SDL_BINARY]


from .__init__ import ctypes, typing, abc, \
    SDL_POINTER, SDL_FUNC, SDL_NET_BINARY

from .SDL_version import SDL_VERSIONNUM

SDL_NET_MAJOR_VERSION, SDL_NET_MINOR_VERSION, SDL_NET_MICRO_VERSION = 3, 0, 0
SDL_NET_VERSION: int = SDL_VERSIONNUM(SDL_NET_MAJOR_VERSION, SDL_NET_MINOR_VERSION, SDL_NET_MICRO_VERSION)

SDL_NET_VERSION_ATLEAST: abc.Callable[[int, int, int], bool] = lambda x, y, z: \
    (SDL_NET_MAJOR_VERSION >= x) and (SDL_NET_MAJOR_VERSION > x or SDL_NET_MINOR_VERSION >= y) and \
        (SDL_NET_MAJOR_VERSION > x or SDL_NET_MINOR_VERSION > y or SDL_NET_MICRO_VERSION >= z)

NET_GetVersion: abc.Callable[..., typing.Any] = SDL_FUNC["NET_GetVersion", ctypes.c_int, [], SDL_NET_BINARY]
NET_Init: abc.Callable[..., typing.Any] = SDL_FUNC["NET_Init", ctypes.c_bool, [], SDL_NET_BINARY]
NET_Quit: abc.Callable[..., typing.Any] = SDL_FUNC["NET_Quit", None, [], SDL_NET_BINARY]

class NET_Address(ctypes.c_void_p):
    ...

NET_ResolveHostname: abc.Callable[..., typing.Any] = SDL_FUNC["NET_ResolveHostname", SDL_POINTER[NET_Address], [ctypes.c_char_p], SDL_NET_BINARY]
NET_WaitUntilResolved: abc.Callable[..., typing.Any] = SDL_FUNC["NET_WaitUntilResolved", ctypes.c_int, [SDL_POINTER[NET_Address], ctypes.c_int32], SDL_NET_BINARY]
NET_GetAddressStatus: abc.Callable[..., typing.Any] = SDL_FUNC["NET_GetAddressStatus", ctypes.c_int, [SDL_POINTER[NET_Address]], SDL_NET_BINARY]
NET_GetAddressString: abc.Callable[..., typing.Any] = SDL_FUNC["NET_GetAddressString", ctypes.c_char_p, [SDL_POINTER[NET_Address]], SDL_NET_BINARY]
NET_RefAddress: abc.Callable[..., typing.Any] = SDL_FUNC["NET_RefAddress", SDL_POINTER[NET_Address], [SDL_POINTER[NET_Address]], SDL_NET_BINARY]
NET_UnrefAddress: abc.Callable[..., typing.Any] = SDL_FUNC["NET_UnrefAddress", None, [SDL_POINTER[NET_Address]], SDL_NET_BINARY]
NET_SimulateAddressResolutionLoss: abc.Callable[..., typing.Any] = SDL_FUNC["NET_SimulateAddressResolutionLoss", None, [ctypes.c_int], SDL_NET_BINARY]
NET_CompareAddresses: abc.Callable[..., typing.Any] = SDL_FUNC["NET_CompareAddresses", ctypes.c_int, [SDL_POINTER[NET_Address], SDL_POINTER[NET_Address]], SDL_NET_BINARY]
NET_GetLocalAddresses: abc.Callable[..., typing.Any] = SDL_FUNC["NET_GetLocalAddresses", SDL_POINTER[SDL_POINTER[NET_Address]], [SDL_POINTER[ctypes.c_int]], SDL_NET_BINARY]
NET_FreeLocalAddresses: abc.Callable[..., typing.Any] = SDL_FUNC["NET_FreeLocalAddresses", None, [SDL_POINTER[SDL_POINTER[NET_Address]]], SDL_NET_BINARY]

class NET_StreamSocket(ctypes.c_void_p):
    ...

NET_CreateClient: abc.Callable[..., typing.Any] = SDL_FUNC["NET_CreateClient", SDL_POINTER[NET_StreamSocket], [SDL_POINTER[NET_Address], ctypes.c_uint16], SDL_NET_BINARY]
NET_WaitUntilConnected: abc.Callable[..., typing.Any] = SDL_FUNC["NET_WaitUntilConnected", ctypes.c_int, [SDL_POINTER[NET_StreamSocket], ctypes.c_int32], SDL_NET_BINARY]

class NET_Server(ctypes.c_void_p):
    ...

NET_CreateServer: abc.Callable[..., typing.Any] = SDL_FUNC["NET_CreateServer", SDL_POINTER[NET_Server], [SDL_POINTER[NET_Address], ctypes.c_uint16], SDL_NET_BINARY]
NET_AcceptClient: abc.Callable[..., typing.Any] = SDL_FUNC["NET_AcceptClient", ctypes.c_bool, [SDL_POINTER[NET_Server], SDL_POINTER[SDL_POINTER[NET_StreamSocket]]], SDL_NET_BINARY]
NET_DestroyServer: abc.Callable[..., typing.Any] = SDL_FUNC["NET_DestroyServer", None, [SDL_POINTER[NET_Server]], SDL_NET_BINARY]
NET_GetStreamSocketAddress: abc.Callable[..., typing.Any] = SDL_FUNC["NET_GetStreamSocketAddress", SDL_POINTER[NET_Address], [SDL_POINTER[NET_StreamSocket]], SDL_NET_BINARY]
NET_GetConnectionStatus: abc.Callable[..., typing.Any] = SDL_FUNC["NET_GetConnectionStatus", ctypes.c_int, [SDL_POINTER[NET_StreamSocket]], SDL_NET_BINARY]
NET_WriteToStreamSocket: abc.Callable[..., typing.Any] = SDL_FUNC["NET_WriteToStreamSocket", ctypes.c_bool, [SDL_POINTER[NET_StreamSocket], ctypes.c_void_p, ctypes.c_int], SDL_NET_BINARY]
NET_GetStreamSocketPendingWrites: abc.Callable[..., typing.Any] = SDL_FUNC["NET_GetStreamSocketPendingWrites", ctypes.c_int, [SDL_POINTER[NET_StreamSocket]], SDL_NET_BINARY]
NET_WaitUntilStreamSocketDrained: abc.Callable[..., typing.Any] = SDL_FUNC["NET_WaitUntilStreamSocketDrained", ctypes.c_int, [SDL_POINTER[NET_StreamSocket], ctypes.c_int32], SDL_NET_BINARY]
NET_ReadFromStreamSocket: abc.Callable[..., typing.Any] = SDL_FUNC["NET_ReadFromStreamSocket", ctypes.c_int, [SDL_POINTER[NET_StreamSocket], ctypes.c_void_p, ctypes.c_int], SDL_NET_BINARY]
NET_SimulateStreamPacketLoss: abc.Callable[..., typing.Any] = SDL_FUNC["NET_SimulateStreamPacketLoss", None, [SDL_POINTER[NET_StreamSocket], ctypes.c_int], SDL_NET_BINARY]
NET_DestroyStreamSocket: abc.Callable[..., typing.Any] = SDL_FUNC["NET_DestroyStreamSocket", None, [SDL_POINTER[NET_StreamSocket]], SDL_NET_BINARY]

class NET_DatagramSocket(ctypes.c_void_p):
    ...

class NET_Datagram(ctypes.Structure):
    _fields_ = [
        ("addr", SDL_POINTER[NET_Address]),
        ("port", ctypes.c_uint16),
        ("buf", SDL_POINTER[ctypes.c_uint8]),
        ("buflen", ctypes.c_int)
    ]

NET_CreateDatagramSocket: abc.Callable[..., typing.Any] = SDL_FUNC["NET_CreateDatagramSocket", SDL_POINTER[NET_DatagramSocket], [SDL_POINTER[NET_Address], ctypes.c_uint16], SDL_NET_BINARY]
NET_SendDatagram: abc.Callable[..., typing.Any] = SDL_FUNC["NET_SendDatagram", ctypes.c_bool, [SDL_POINTER[NET_DatagramSocket], SDL_POINTER[NET_Address], ctypes.c_uint16, ctypes.c_void_p, ctypes.c_int], SDL_NET_BINARY]
NET_ReceiveDatagram: abc.Callable[..., typing.Any] = SDL_FUNC["NET_ReceiveDatagram", ctypes.c_bool, [SDL_POINTER[NET_DatagramSocket], SDL_POINTER[SDL_POINTER[NET_Datagram]]], SDL_NET_BINARY]
NET_DestroyDatagram: abc.Callable[..., typing.Any] = SDL_FUNC["NET_DestroyDatagram", None, [SDL_POINTER[NET_Datagram]], SDL_NET_BINARY]
NET_SimulateDatagramPacketLoss: abc.Callable[..., typing.Any] = SDL_FUNC["NET_SimulateDatagramPacketLoss", None, [SDL_POINTER[NET_DatagramSocket], ctypes.c_int], SDL_NET_BINARY]
NET_DestroyDatagramSocket: abc.Callable[..., typing.Any] = SDL_FUNC["NET_DestroyDatagramSocket", None, [SDL_POINTER[NET_DatagramSocket]], SDL_NET_BINARY]
NET_WaitUntilInputAvailable: abc.Callable[..., typing.Any] = SDL_FUNC["NET_WaitUntilInputAvailable", ctypes.c_int, [SDL_POINTER[ctypes.c_void_p], ctypes.c_int, ctypes.c_int32], SDL_NET_BINARY]


from .__init__ import ctypes, typing, abc, \
    SDL_POINTER, SDL_FUNC, SDL_TYPE, SDL_BINARY

SDL_SpinLock: typing.TypeAlias = SDL_TYPE["SDL_SpinLock", ctypes.c_int]

SDL_TryLockSpinlock: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_TryLockSpinlock", ctypes.c_bool, [SDL_POINTER[SDL_SpinLock]], SDL_BINARY]
SDL_LockSpinlock: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_LockSpinlock", None, [SDL_POINTER[SDL_SpinLock]], SDL_BINARY]
SDL_UnlockSpinlock: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_UnlockSpinlock", None, [SDL_POINTER[SDL_SpinLock]], SDL_BINARY]

SDL_MemoryBarrierReleaseFunction: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_MemoryBarrierReleaseFunction", None, [], SDL_BINARY]
SDL_MemoryBarrierAcquireFunction: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_MemoryBarrierAcquireFunction", None, [], SDL_BINARY]

class SDL_AtomicInt(ctypes.Structure):
    _fields_ = [
        ("value", ctypes.c_int)
    ]

SDL_CompareAndSwapAtomicInt: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_CompareAndSwapAtomicInt", ctypes.c_bool, [SDL_POINTER[SDL_AtomicInt], ctypes.c_int, ctypes.c_int], SDL_BINARY]
SDL_SetAtomicInt: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetAtomicInt", ctypes.c_int, [SDL_POINTER[SDL_AtomicInt], ctypes.c_int], SDL_BINARY]
SDL_GetAtomicInt: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetAtomicInt", ctypes.c_int, [SDL_POINTER[SDL_AtomicInt]], SDL_BINARY]
SDL_AddAtomicInt: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_AddAtomicInt", ctypes.c_int, [SDL_POINTER[SDL_AtomicInt], ctypes.c_int], SDL_BINARY]

class LP_SDL_AtomicInt(ctypes._Pointer):
    ...

SDL_AtomicIncRef: abc.Callable[[SDL_POINTER[SDL_AtomicInt]], ctypes.c_int] = lambda a: SDL_AddAtomicInt(a, 1)
SDL_AtomicDecRef: abc.Callable[[SDL_POINTER[SDL_AtomicInt]], bool] = lambda a: SDL_AddAtomicInt(a, -1) == 1

class SDL_AtomicU32(ctypes.Structure):
    _fields_ = [
        ("value", ctypes.c_uint32)
    ]

SDL_CompareAndSwapAtomicU32: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_CompareAndSwapAtomicU32", ctypes.c_bool, [SDL_POINTER[SDL_AtomicU32], ctypes.c_uint32, ctypes.c_uint32], SDL_BINARY]
SDL_SetAtomicU32: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetAtomicU32", ctypes.c_uint32, [SDL_POINTER[SDL_AtomicU32], ctypes.c_uint32], SDL_BINARY]
SDL_GetAtomicU32: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetAtomicU32", ctypes.c_uint32, [SDL_POINTER[SDL_AtomicU32]], SDL_BINARY]

SDL_CompareAndSwapAtomicPointer: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_CompareAndSwapAtomicPointer", ctypes.c_bool, [SDL_POINTER[ctypes.c_void_p], ctypes.c_void_p, ctypes.c_void_p], SDL_BINARY]
SDL_SetAtomicPointer: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetAtomicPointer", ctypes.c_void_p, [SDL_POINTER[ctypes.c_void_p], ctypes.c_void_p], SDL_BINARY]
SDL_GetAtomicPointer: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetAtomicPointer", ctypes.c_void_p, [SDL_POINTER[ctypes.c_void_p]], SDL_BINARY]


from .__init__ import ctypes, typing, abc, SDL_POINTER, \
    SDL_FUNC_TYPE, SDL_FUNC, SDL_TYPE, SDL_BINARY, SDL_ENUM

from .SDL_sensor import SDL_SensorType
from .SDL_properties import SDL_PropertiesID
from .SDL_power import SDL_PowerState
from .SDL_guid import SDL_GUID

class SDL_Joystick(ctypes.c_void_p):
    ...

SDL_JoystickID: typing.TypeAlias = SDL_TYPE["SDL_JoystickID", ctypes.c_uint32]
SDL_JoystickType: typing.TypeAlias = SDL_TYPE["SDL_JoystickType", SDL_ENUM]

SDL_JOYSTICK_TYPE_UNKNOWN, SDL_JOYSTICK_TYPE_GAMEPAD, SDL_JOYSTICK_TYPE_WHEEL , SDL_JOYSTICK_TYPE_ARCADE_, \
    SDL_JOYSTICK_TYPE_FLIGHT_, SDL_JOYSTICK_TYPE_DANCE_P, SDL_JOYSTICK_TYPE_GUITAR , SDL_JOYSTICK_TYPE_DRUM_KI, \
        SDL_JOYSTICK_TYPE_ARCADE_, SDL_JOYSTICK_TYPE_THROTTL, SDL_JOYSTICK_TYPE_COUNT = range(11)

SDL_JoystickConnectionState: typing.TypeAlias = SDL_TYPE["SDL_JoystickConnectionState", SDL_ENUM]

SDL_JOYSTICK_CONNECTION_INVALID, SDL_JOYSTICK_CONNECTION_UNKNOWN, \
    SDL_JOYSTICK_CONNECTION_WIRED, SDL_JOYSTICK_CONNECTION_WIRELESS = range(-1, 3)

SDL_JOYSTICK_AXIS_MAX, SDL_JOYSTICK_AXIS_MIN = 32767, -32768

SDL_LockJoysticks: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_LockJoysticks", None, [], SDL_BINARY]
SDL_UnlockJoysticks: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_UnlockJoysticks", None, [], SDL_BINARY]
SDL_HasJoystick: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_HasJoystick", ctypes.c_bool, [], SDL_BINARY]
SDL_GetJoysticks: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetJoysticks", SDL_POINTER[SDL_JoystickID], [SDL_POINTER[ctypes.c_int]], SDL_BINARY]
SDL_GetJoystickNameForID: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetJoystickNameForID", ctypes.c_char_p, [SDL_JoystickID], SDL_BINARY]
SDL_GetJoystickPathForID: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetJoystickPathForID", ctypes.c_char_p, [SDL_JoystickID], SDL_BINARY]
SDL_GetJoystickPlayerIndexForID: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetJoystickPlayerIndexForID", ctypes.c_int, [SDL_JoystickID], SDL_BINARY]
SDL_GetJoystickGUIDForID: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetJoystickGUIDForID", SDL_GUID, [SDL_JoystickID], SDL_BINARY]
SDL_GetJoystickVendorForID: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetJoystickVendorForID", ctypes.c_uint16, [SDL_JoystickID], SDL_BINARY]
SDL_GetJoystickProductForID: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetJoystickProductForID", ctypes.c_uint16, [SDL_JoystickID], SDL_BINARY]
SDL_GetJoystickProductVersionForID: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetJoystickProductVersionForID", ctypes.c_uint16, [SDL_JoystickID], SDL_BINARY]
SDL_GetJoystickTypeForID: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetJoystickTypeForID", SDL_JoystickType, [SDL_JoystickID], SDL_BINARY]
SDL_OpenJoystick: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_OpenJoystick", SDL_POINTER[SDL_Joystick], [SDL_JoystickID], SDL_BINARY]
SDL_GetJoystickFromID: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetJoystickFromID", SDL_POINTER[SDL_Joystick], [SDL_JoystickID], SDL_BINARY]
SDL_GetJoystickFromPlayerIndex: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetJoystickFromPlayerIndex", SDL_POINTER[SDL_Joystick], [ctypes.c_int], SDL_BINARY]

class SDL_VirtualJoystickTouchpadDesc(ctypes.Structure):
    _fields_ = [
        ("nfingers", ctypes.c_uint16),
        ("padding", ctypes.c_uint16 * 3)
    ]

class SDL_VirtualJoystickSensorDesc(ctypes.Structure):
    _fields_ = [
        ("type", SDL_SensorType),
        ("rate", ctypes.c_float)
    ]

class SDL_VirtualJoystickDesc(ctypes.Structure):
    _fields_ = [
        ("version", ctypes.c_uint32),
        ("type", ctypes.c_uint16),
        ("padding", ctypes.c_uint16),
        ("vendor_id", ctypes.c_uint16),
        ("product_id", ctypes.c_uint16),
        ("naxes", ctypes.c_uint16),
        ("nbuttons", ctypes.c_uint16),
        ("nballs", ctypes.c_uint16),
        ("nhats", ctypes.c_uint16),
        ("ntouchpads", ctypes.c_uint16),
        ("nsensors", ctypes.c_uint16),
        ("padding2", ctypes.c_uint16 * 2),
        ("button_mask", ctypes.c_uint32),
        ("axis_mask", ctypes.c_uint32),
        ("name", ctypes.c_char_p),
        ("touchpads", SDL_POINTER[SDL_VirtualJoystickTouchpadDesc]),
        ("sensors", SDL_POINTER[SDL_VirtualJoystickSensorDesc]),
        ("userdata", ctypes.c_void_p),
        ("Update", SDL_FUNC_TYPE["SDL_VirtualJoystickDesc.Update", None, [ctypes.c_void_p]]),
        ("SetPlayerIndex", SDL_FUNC_TYPE["SDL_VirtualJoystickDesc.SetPlayerIndex", None, [ctypes.c_void_p, ctypes.c_int]]),
        ("Rumble", SDL_FUNC_TYPE["SDL_VirtualJoystickDesc.Rumble", ctypes.c_bool, [ctypes.c_void_p, ctypes.c_uint16, ctypes.c_uint16]]),
        ("RumbleTriggers", SDL_FUNC_TYPE["SDL_VirtualJoystickDesc.RumbleTriggers", ctypes.c_bool, [ctypes.c_void_p, ctypes.c_uint16, ctypes.c_uint16]]),
        ("SetLED", SDL_FUNC_TYPE["SDL_VirtualJoystickDesc.SetLED", ctypes.c_bool, [ctypes.c_void_p, ctypes.c_uint8, ctypes.c_uint8, ctypes.c_uint8]]),
        ("SendEffect", SDL_FUNC_TYPE["SDL_VirtualJoystickDesc.SendEffect", ctypes.c_bool, [ctypes.c_void_p, ctypes.c_void_p, ctypes.c_int]]),
        ("SetSensorsEnabled", SDL_FUNC_TYPE["SDL_VirtualJoystickDesc.SetSensorsEnabled", ctypes.c_bool, [ctypes.c_void_p, ctypes.c_bool]]),
        ("Cleanup", SDL_FUNC_TYPE["SDL_VirtualJoystickDesc.Cleanup", None, [ctypes.c_void_p]])
    ]

SDL_AttachVirtualJoystick: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_AttachVirtualJoystick", SDL_JoystickID, [SDL_POINTER[SDL_VirtualJoystickDesc]], SDL_BINARY]
SDL_DetachVirtualJoystick: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_DetachVirtualJoystick", ctypes.c_bool, [SDL_JoystickID], SDL_BINARY]
SDL_IsJoystickVirtual: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_IsJoystickVirtual", ctypes.c_bool, [SDL_JoystickID], SDL_BINARY]
SDL_SetJoystickVirtualAxis: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetJoystickVirtualAxis", ctypes.c_bool, [SDL_POINTER[SDL_Joystick], ctypes.c_int, ctypes.c_int16], SDL_BINARY]
SDL_SetJoystickVirtualBall: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetJoystickVirtualBall", ctypes.c_bool, [SDL_POINTER[SDL_Joystick], ctypes.c_int, ctypes.c_int16, ctypes.c_int16], SDL_BINARY]
SDL_SetJoystickVirtualButton: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetJoystickVirtualButton", ctypes.c_bool, [SDL_POINTER[SDL_Joystick], ctypes.c_int, ctypes.c_bool], SDL_BINARY]
SDL_SetJoystickVirtualHat: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetJoystickVirtualHat", ctypes.c_bool, [SDL_POINTER[SDL_Joystick], ctypes.c_int, ctypes.c_uint8], SDL_BINARY]
SDL_SetJoystickVirtualTouchpad: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetJoystickVirtualTouchpad", ctypes.c_bool, [SDL_POINTER[SDL_Joystick], ctypes.c_int, ctypes.c_int, ctypes.c_bool, ctypes.c_float, ctypes.c_float, ctypes.c_float], SDL_BINARY]
SDL_SendJoystickVirtualSensorData: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SendJoystickVirtualSensorData", ctypes.c_bool, [SDL_POINTER[SDL_Joystick], SDL_SensorType, ctypes.c_uint64, SDL_POINTER[ctypes.c_float], ctypes.c_int], SDL_BINARY]
SDL_GetJoystickProperties: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetJoystickProperties", SDL_PropertiesID, [SDL_POINTER[SDL_Joystick]], SDL_BINARY]

SDL_PROP_JOYSTICK_CAP_MONO_LED_BOOLEAN: bytes = "SDL.joystick.cap.mono_led".encode()
SDL_PROP_JOYSTICK_CAP_RGB_LED_BOOLEAN: bytes = "SDL.joystick.cap.rgb_led".encode()
SDL_PROP_JOYSTICK_CAP_PLAYER_LED_BOOLEAN: bytes = "SDL.joystick.cap.player_led".encode()
SDL_PROP_JOYSTICK_CAP_RUMBLE_BOOLEAN: bytes = "SDL.joystick.cap.rumble".encode()
SDL_PROP_JOYSTICK_CAP_TRIGGER_RUMBLE_BOOLEAN: bytes = "SDL.joystick.cap.trigger_rumble".encode()

SDL_GetJoystickName: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetJoystickName", ctypes.c_char_p, [SDL_POINTER[SDL_Joystick]], SDL_BINARY]
SDL_GetJoystickPath: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetJoystickPath", ctypes.c_char_p, [SDL_POINTER[SDL_Joystick]], SDL_BINARY]
SDL_GetJoystickPlayerIndex: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetJoystickPlayerIndex", ctypes.c_int, [SDL_POINTER[SDL_Joystick]], SDL_BINARY]
SDL_SetJoystickPlayerIndex: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetJoystickPlayerIndex", ctypes.c_bool, [SDL_POINTER[SDL_Joystick], ctypes.c_int], SDL_BINARY]
SDL_GetJoystickGUID: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetJoystickGUID", SDL_GUID, [SDL_POINTER[SDL_Joystick]], SDL_BINARY]
SDL_GetJoystickVendor: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetJoystickVendor", ctypes.c_uint16, [SDL_POINTER[SDL_Joystick]], SDL_BINARY]
SDL_GetJoystickProduct: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetJoystickProduct", ctypes.c_uint16, [SDL_POINTER[SDL_Joystick]], SDL_BINARY]
SDL_GetJoystickProductVersion: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetJoystickProductVersion", ctypes.c_uint16, [SDL_POINTER[SDL_Joystick]], SDL_BINARY]
SDL_GetJoystickFirmwareVersion: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetJoystickFirmwareVersion", ctypes.c_uint16, [SDL_POINTER[SDL_Joystick]], SDL_BINARY]
SDL_GetJoystickSerial: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetJoystickSerial", ctypes.c_char_p, [SDL_POINTER[SDL_Joystick]], SDL_BINARY]
SDL_GetJoystickType: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetJoystickType", SDL_JoystickType, [SDL_POINTER[SDL_Joystick]], SDL_BINARY]
SDL_GetJoystickGUIDInfo: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetJoystickGUIDInfo", None, [SDL_GUID, SDL_POINTER[ctypes.c_uint16], SDL_POINTER[ctypes.c_uint16], SDL_POINTER[ctypes.c_uint16], SDL_POINTER[ctypes.c_uint16]], SDL_BINARY]
SDL_JoystickConnected: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_JoystickConnected", ctypes.c_bool, [SDL_POINTER[SDL_Joystick]], SDL_BINARY]
SDL_GetJoystickID: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetJoystickID", SDL_JoystickID, [SDL_POINTER[SDL_Joystick]], SDL_BINARY]
SDL_GetNumJoystickAxes: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetNumJoystickAxes", ctypes.c_int, [SDL_POINTER[SDL_Joystick]], SDL_BINARY]
SDL_GetNumJoystickBalls: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetNumJoystickBalls", ctypes.c_int, [SDL_POINTER[SDL_Joystick]], SDL_BINARY]
SDL_GetNumJoystickHats: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetNumJoystickHats", ctypes.c_int, [SDL_POINTER[SDL_Joystick]], SDL_BINARY]
SDL_GetNumJoystickButtons: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetNumJoystickButtons", ctypes.c_int, [SDL_POINTER[SDL_Joystick]], SDL_BINARY]
SDL_SetJoystickEventsEnabled: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetJoystickEventsEnabled", None, [ctypes.c_bool], SDL_BINARY]
SDL_JoystickEventsEnabled: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_JoystickEventsEnabled", ctypes.c_bool, [], SDL_BINARY]
SDL_UpdateJoysticks: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_UpdateJoysticks", None, [], SDL_BINARY]
SDL_GetJoystickAxis: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetJoystickAxis", ctypes.c_int16, [SDL_POINTER[SDL_Joystick], ctypes.c_int], SDL_BINARY]
SDL_GetJoystickAxisInitialState: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetJoystickAxisInitialState", ctypes.c_bool, [SDL_POINTER[SDL_Joystick], ctypes.c_int, SDL_POINTER[ctypes.c_int16]], SDL_BINARY]
SDL_GetJoystickBall: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetJoystickBall", ctypes.c_bool, [SDL_POINTER[SDL_Joystick], ctypes.c_int, SDL_POINTER[ctypes.c_int], SDL_POINTER[ctypes.c_int]], SDL_BINARY]
SDL_GetJoystickHat: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetJoystickHat", ctypes.c_uint8, [SDL_POINTER[SDL_Joystick], ctypes.c_int], SDL_BINARY]

SDL_HAT_CENTERED, SDL_HAT_UP, SDL_HAT_RIGHT, SDL_HAT_DOWN, SDL_HAT_LEFT = 0x00, 0x01, 0x02, 0x04, 0x08

SDL_HAT_RIGHTUP, SDL_HAT_RIGHTDOWN, SDL_HAT_LEFTUP, SDL_HAT_LEFTDOWN = \
    SDL_HAT_RIGHT | SDL_HAT_UP, SDL_HAT_RIGHT | SDL_HAT_DOWN, SDL_HAT_LEFT | SDL_HAT_UP, SDL_HAT_LEFT | SDL_HAT_DOWN

SDL_GetJoystickButton: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetJoystickButton", ctypes.c_bool, [SDL_POINTER[SDL_Joystick], ctypes.c_int], SDL_BINARY]
SDL_RumbleJoystick: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_RumbleJoystick", ctypes.c_bool, [SDL_POINTER[SDL_Joystick], ctypes.c_uint16, ctypes.c_uint16, ctypes.c_uint32], SDL_BINARY]
SDL_RumbleJoystickTriggers: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_RumbleJoystickTriggers", ctypes.c_bool, [SDL_POINTER[SDL_Joystick], ctypes.c_uint16, ctypes.c_uint16, ctypes.c_uint32], SDL_BINARY]
SDL_SetJoystickLED: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetJoystickLED", ctypes.c_bool, [SDL_POINTER[SDL_Joystick], ctypes.c_uint8, ctypes.c_uint8, ctypes.c_uint8], SDL_BINARY]
SDL_SendJoystickEffect: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SendJoystickEffect", ctypes.c_bool, [SDL_POINTER[SDL_Joystick], ctypes.c_void_p, ctypes.c_int], SDL_BINARY]
SDL_CloseJoystick: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_CloseJoystick", None, [SDL_POINTER[SDL_Joystick]], SDL_BINARY]
SDL_GetJoystickConnectionState: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetJoystickConnectionState", SDL_JoystickConnectionState, [SDL_POINTER[SDL_Joystick]], SDL_BINARY]
SDL_GetJoystickPowerInfo: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetJoystickPowerInfo", SDL_PowerState, [SDL_POINTER[SDL_Joystick], SDL_POINTER[ctypes.c_int]], SDL_BINARY]


from .__init__ import ctypes, typing, abc, SDL_POINTER, \
    SDL_FUNC_TYPE, SDL_FUNC, SDL_TYPE, SDL_BINARY, SDL_ENUM

from .SDL_stdinc import SDL_Time

SDL_GetBasePath: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetBasePath", ctypes.c_char_p, [], SDL_BINARY]
SDL_GetPrefPath: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetPrefPath", ctypes.c_char_p, [ctypes.c_char_p, ctypes.c_char_p], SDL_BINARY]

SDL_Folder: typing.TypeAlias = SDL_TYPE["SDL_Folder", SDL_ENUM]

SDL_FOLDER_HOME, SDL_FOLDER_DESKTOP, SDL_FOLDER_DOCUMENTS, SDL_FOLDER_DOWNLOADS, SDL_FOLDER_MUSIC, SDL_FOLDER_PICTURES, SDL_FOLDER_PUBLICSHARE, \
    SDL_FOLDER_SAVEDGAMES, SDL_FOLDER_SCREENSHOTS, SDL_FOLDER_TEMPLATES, SDL_FOLDER_VIDEOS , SDL_FOLDER_COUNT = range(12)

SDL_GetUserFolder: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetUserFolder", ctypes.c_char_p, [SDL_Folder], SDL_BINARY]

SDL_PathType: typing.TypeAlias = SDL_TYPE["SDL_PathType", SDL_ENUM]

SDL_PATHTYPE_NONE, SDL_PATHTYPE_FILE, SDL_PATHTYPE_DIRECTORY, SDL_PATHTYPE_OTHER = range(4)

class SDL_PathInfo(ctypes.Structure):
    _fields_ = [
        ("type", SDL_PathType),
        ("size", ctypes.c_uint64),
        ("create_time", SDL_Time),
        ("modify_time", SDL_Time),
        ("access_time", SDL_Time)
    ]

SDL_GlobFlags: typing.TypeAlias = SDL_TYPE["SDL_GlobFlags", ctypes.c_uint32]

SDL_GLOB_CASEINSENSITIVE: int = 1 << 0

SDL_CreateDirectory: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_CreateDirectory", ctypes.c_bool, [ctypes.c_char_p], SDL_BINARY]

SDL_EnumerationResult: typing.TypeAlias = SDL_TYPE["SDL_EnumerationResult", SDL_ENUM]

SDL_ENUM_CONTINUE, SDL_ENUM_SUCCESS, SDL_ENUM_FAILURE = range(3)

SDL_EnumerateDirectoryCallback: typing.TypeAlias = SDL_FUNC_TYPE["SDL_EnumerateDirectoryCallback", SDL_EnumerationResult, [ctypes.c_void_p, ctypes.c_char_p, ctypes.c_char_p]]

SDL_EnumerateDirectory: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_EnumerateDirectory", ctypes.c_bool, [ctypes.c_char_p, SDL_EnumerateDirectoryCallback, ctypes.c_void_p], SDL_BINARY]
SDL_RemovePath: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_RemovePath", ctypes.c_bool, [ctypes.c_char_p], SDL_BINARY]
SDL_RenamePath: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_RenamePath", ctypes.c_bool, [ctypes.c_char_p, ctypes.c_char_p], SDL_BINARY]
SDL_CopyFile: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_CopyFile", ctypes.c_bool, [ctypes.c_char_p, ctypes.c_char_p], SDL_BINARY]
SDL_GetPathInfo: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetPathInfo", ctypes.c_bool, [ctypes.c_char_p, SDL_POINTER[SDL_PathInfo]], SDL_BINARY]
SDL_GlobDirectory: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GlobDirectory", SDL_POINTER[ctypes.c_char_p], [ctypes.c_char_p, ctypes.c_char_p, SDL_GlobFlags, SDL_POINTER[ctypes.c_int]], SDL_BINARY]
SDL_GetCurrentDirectory: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetCurrentDirectory", ctypes.c_char_p, [], SDL_BINARY]


from .__init__ import ctypes, typing, abc, \
    SDL_POINTER, SDL_FUNC, SDL_TYPE, SDL_BINARY

from .SDL_video import SDL_Window
from .SDL_stdinc import SDL_FunctionPointer

class VkInstance(ctypes.c_void_p):
    ...

class VkPhysicalDevice(ctypes.c_void_p):
    ...

class VkAllocationCallbacks(ctypes.c_void_p):
    ...
    
VkSurfaceKHR: typing.TypeAlias = SDL_TYPE["VkSurfaceKHR", ctypes.c_uint64]

SDL_Vulkan_LoadLibrary: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_Vulkan_LoadLibrary", ctypes.c_bool, [ctypes.c_char_p], SDL_BINARY]
SDL_Vulkan_GetVkGetInstanceProcAddr: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_Vulkan_GetVkGetInstanceProcAddr", SDL_FunctionPointer, [], SDL_BINARY]
SDL_Vulkan_UnloadLibrary: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_Vulkan_UnloadLibrary", None, [], SDL_BINARY]
SDL_Vulkan_GetInstanceExtensions: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_Vulkan_GetInstanceExtensions", SDL_POINTER[ctypes.c_char_p], [SDL_POINTER[ctypes.c_uint32]], SDL_BINARY]

SDL_Vulkan_CreateSurface: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_Vulkan_CreateSurface", ctypes.c_bool, [SDL_POINTER[SDL_Window], VkInstance, SDL_POINTER[VkAllocationCallbacks], SDL_POINTER[VkSurfaceKHR]], SDL_BINARY]
SDL_Vulkan_DestroySurface: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_Vulkan_DestroySurface", None, [VkInstance, VkSurfaceKHR, SDL_POINTER[VkAllocationCallbacks]], SDL_BINARY]
SDL_Vulkan_GetPresentationSupport: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_Vulkan_GetPresentationSupport", ctypes.c_bool, [VkInstance, VkPhysicalDevice, ctypes.c_uint32], SDL_BINARY]


from .__init__ import ctypes, typing, abc, \
    SDL_POINTER, SDL_FUNC, SDL_TYPE, SDL_BINARY

from .SDL_video import SDL_Window

SDL_MetalView: typing.TypeAlias = SDL_TYPE["SDL_MetalView", ctypes.c_void_p]

SDL_Metal_CreateView: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_Metal_CreateView", SDL_MetalView, [SDL_POINTER[SDL_Window]], SDL_BINARY]
SDL_Metal_DestroyView: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_Metal_DestroyView", None, [SDL_MetalView], SDL_BINARY]
SDL_Metal_GetLayer: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_Metal_GetLayer", ctypes.c_void_p, [SDL_MetalView], SDL_BINARY]


from .__init__ import ctypes, typing, abc, SDL_FUNC, SDL_BINARY

SDL_CACHELINE_SIZE: int = 128

SDL_GetNumLogicalCPUCores: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetNumLogicalCPUCores", ctypes.c_int, [], SDL_BINARY]
SDL_GetCPUCacheLineSize: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetCPUCacheLineSize", ctypes.c_int, [], SDL_BINARY]

SDL_HasAltiVec: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_HasAltiVec", ctypes.c_bool, [], SDL_BINARY]
SDL_HasMMX: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_HasMMX", ctypes.c_bool, [], SDL_BINARY]
SDL_HasSSE: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_HasSSE", ctypes.c_bool, [], SDL_BINARY]
SDL_HasSSE2: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_HasSSE2", ctypes.c_bool, [], SDL_BINARY]
SDL_HasSSE3: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_HasSSE3", ctypes.c_bool, [], SDL_BINARY]
SDL_HasSSE41: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_HasSSE41", ctypes.c_bool, [], SDL_BINARY]
SDL_HasSSE42: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_HasSSE42", ctypes.c_bool, [], SDL_BINARY]
SDL_HasAVX: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_HasAVX", ctypes.c_bool, [], SDL_BINARY]
SDL_HasAVX2: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_HasAVX2", ctypes.c_bool, [], SDL_BINARY]
SDL_HasAVX512F: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_HasAVX512F", ctypes.c_bool, [], SDL_BINARY]
SDL_HasARMSIMD: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_HasARMSIMD", ctypes.c_bool, [], SDL_BINARY]
SDL_HasNEON: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_HasNEON", ctypes.c_bool, [], SDL_BINARY]
SDL_HasLSX: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_HasLSX", ctypes.c_bool, [], SDL_BINARY]
SDL_HasLASX: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_HasLASX", ctypes.c_bool, [], SDL_BINARY]

SDL_GetSystemRAM: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetSystemRAM", ctypes.c_int, [], SDL_BINARY]
SDL_GetSIMDAlignment: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetSIMDAlignment", ctypes.c_size_t, [], SDL_BINARY]